@@ -1,16 +1,21 @@
 """Console script for covid19_outbreak_simulator."""
 import argparse
 import sys
+import shlex
+import subprocess
 import multiprocessing
 import numpy as np
 from io import StringIO
 from tqdm import tqdm
 from collections import defaultdict
+from datetime import datetime
+from importlib import import_module
+
 from .simulator import Simulator
 from .model import Params
 
 
-def summarize_simulations(args):
+def summarize_simulations(logfile):
     # write some summary information into standard output
     n_simulation = 0
     total_infection = 0
@@ -19,6 +24,7 @@ def summarize_simulations(args):
     total_infection_ignored = 0
     total_show_symptom = 0
     total_removal = 0
+    total_recover = 0
     total_quarantine = 0
     total_reintegration = 0
     total_abort = 0
@@ -40,7 +46,10 @@ def summarize_simulations(args):
     n_second_symptom_on_day = defaultdict(int)
     n_third_symptom_on_day = defaultdict(int)
     #
-    with open(args.logfile) as lines:
+    timed_stats = defaultdict(dict)
+    customized_stats = defaultdict(dict)
+    #
+    with open(logfile) as lines:
         infection_from_seed_per_sim = defaultdict(int)
         infection_time_from_seed_per_sim = defaultdict(int)
         first_infection_day_per_sim = defaultdict(int)
@@ -48,13 +57,19 @@ def summarize_simulations(args):
         second_symptom_per_sim = defaultdict(int)
         third_symptom_per_sim = defaultdict(int)
 
-        if args.infectors:
-            infectors = args.infectors
-        else:
-            infectors = ['0']
+        total_popsize = 0
+        infectors = []
+        args = None
 
-        total_popsize = sum(int(x.split('=')[-1]) for x in args.popsize)
         for line in lines:
+            if line.startswith('#'):
+                if line.startswith('# CMD:'):
+                    args = parse_args(shlex.split(line[6:])[1:])
+                    if args.infectors:
+                        infectors = args.infectors
+                    total_popsize = sum(
+                        int(x.split('=')[-1]) for x in args.popsize)
+                continue
             id, time, event, target, params = line.split('\t')
             if id == 'id':
                 # skip header
@@ -88,8 +103,6 @@ def summarize_simulations(args):
                 total_show_symptom += 1
                 if target in infectors:
                     n_seed_show_symptom_on_day[int(time) + 1] += 1
-                if args.pre_quarantine and time < float(args.pre_quarantine[0]):
-                    pass
                 elif id not in first_symptom_day_per_sim:
                     first_symptom_day_per_sim[id] = time
                 elif id not in second_symptom_per_sim:
@@ -109,36 +122,56 @@ def summarize_simulations(args):
             elif event == 'END':
                 n_simulation += 1
                 n_remaining_popsize[target] += 1
-                if not args.pre_quarantine:
-                    n_outbreak_duration[0 if time == 0 else int(time) + 1] += 1
-                else:
-                    n_outbreak_duration[
-                        0 if time <= float(args.pre_quarantine[0]) else
-                        int(time - float(args.pre_quarantine[0])) + 1] += 1
+                n_outbreak_duration[0 if time == 0 else int(time) + 1] += 1
                 if target == total_popsize and id not in first_symptom_day_per_sim:
                     n_no_outbreak += 1
+            elif event == 'RECOVER':
+                total_recover += 1
+            elif event == 'STAT':
+                id, time, event, target, params = line.split('\t')
+                params = params.split(',')
+                for param in params:
+                    try:
+                        key, value = param.split('=')
+                        if time in timed_stats[key]:
+                            timed_stats[key][time] += ', ' + value.strip()
+                        else:
+                            timed_stats[key][time] = value.strip()
+                    except Exception:
+                        pass
+            elif event == 'ERROR':
+                # an error has happened
+                # params = line.split('\t')[-1]
+                # err = eval(params.split('=', 1)[-1])
+                sys.exit(f'Simulator exists with error')
+
             else:
-                raise ValueError(f'Unrecognized event {event}')
+                # customized events
+                id, time, event, target, params = line.split('\t')
+                params = params.split(',')
+                for param in params:
+                    try:
+                        key, value = param.split('=')
+                        if time in customized_stats[f'{event.lower()}_{key}']:
+                            customized_stats[f'{event.lower()}_{key}'][
+                                time] += ', ' + value.strip()
+                        else:
+                            customized_stats[f'{event.lower()}_{key}'][
+                                time] = value.strip()
+                    except Exception:
+                        pass
+
     # summarize
     for v in infection_from_seed_per_sim.values():
         n_num_infected_by_seed[v] += 1
     #
     for v in infection_time_from_seed_per_sim.values():
-        if args.pre_quarantine:
-            n_first_infected_by_seed_on_day[int(v -
-                                                float(args.pre_quarantine[0])) +
-                                            1] += 1
-        else:
-            n_first_infected_by_seed_on_day[int(v) + 1] += 1
+        n_first_infected_by_seed_on_day[int(v) + 1] += 1
     n_first_infected_by_seed_on_day[0] = n_simulation - sum(
         n_first_infected_by_seed_on_day.values())
     #
     for v in first_infection_day_per_sim.values():
-        if args.pre_quarantine:
-            n_first_infection_on_day[int(v - float(args.pre_quarantine[0])) +
-                                     1] += 1
-        else:
-            n_first_infection_on_day[int(v) + 1] += 1
+        n_first_infection_on_day[int(v) + 1] += 1
     n_first_infection_on_day[0] = n_simulation - sum(
         n_first_infection_on_day.values())
     #
@@ -146,11 +179,7 @@ def summarize_simulations(args):
         n_seed_show_symptom_on_day.values())
     #
     for v in first_symptom_day_per_sim.values():
-        if args.pre_quarantine:
-            n_first_symptom_on_day[int(v - float(args.pre_quarantine[0])) +
-                                   1] += 1
-        else:
-            n_first_symptom_on_day[int(v) + 1] += 1
+        n_first_symptom_on_day[int(v) + 1] += 1
     n_first_symptom = sum(n_first_symptom_on_day.values())
     #
     for v in second_symptom_per_sim.values():
@@ -164,7 +193,7 @@ def summarize_simulations(args):
     # print
     print(f'logfile\t{args.logfile}')
     print(f'popsize\t{" ".join(args.popsize)}')
-    print(f'keep_symptomatic\t{"yes" if args.keep_symptomatic else "no"}')
+    print(f'handle_symptomatic\t{" ".join(args.handle_symptomatic)}')
     if args.symptomatic_r0:
         print(f'symptomatic_r0\t{" - ".join(args.symptomatic_r0)}')
     if args.asymptomatic_r0:
@@ -173,10 +202,6 @@ def summarize_simulations(args):
         print(
             f'incubation_period\t{args.incubation_period[0]}({args.incubation_period[1]}, {args.incubation_period[2]}) '
             + ' '.join(args.incubation_period[3:]))
-    if args.pre_quarantine is None:
-        print(f'pre_quarantine\tno')
-    else:
-        print(f'pre_quarantine\t{" ".join(args.pre_quarantine)}')
     if args.interval == 1 / 24:
         print(f'interval\t1 hour')
     elif args.interval == 1:
@@ -200,6 +225,7 @@ def summarize_simulations(args):
     print(f'total_infection_ignored\t{total_infection_ignored}')
     print(f'total_show_symptom\t{total_show_symptom}')
     print(f'total_removal\t{total_removal}')
+    print(f'total_recover\t{total_recover}')
     print(f'total_quarantine\t{total_quarantine}')
     print(f'total_reintegration\t{total_reintegration}')
     print(f'total_abort\t{total_abort}')
@@ -247,6 +273,38 @@ def summarize_simulations(args):
     print(f'n_third_symptom\t{n_third_symptom}')
     for day in sorted(n_third_symptom_on_day.keys()):
         print(f'n_third_symptom_on_day_{day}\t{n_third_symptom_on_day[day]}')
+    for item, timed_value in timed_stats.items():
+        for time, value in timed_value.items():
+            values = [x.strip() for x in value.split(',')]
+            if len(set(values)) == 1:
+                value = f'[{values[0]}] * {len(values)}'
+                print(f'{item}_{time}\t{value}')
+            else:
+                print(f'{item}_{time}\t{value.strip()}')
+        for time, value in timed_value.items():
+            try:
+                value = eval(value)
+                if len(value) > 1:
+                    value = '{:.3f}'.format(sum(value) / len(value))
+                    print(f'avg_{item}_{time}\t{value}')
+            except:
+                pass
+    for item, timed_value in customized_stats.items():
+        for time, value in timed_value.items():
+            values = [x.strip() for x in value.split(',')]
+            if len(set(values)) == 1:
+                value = f'[{values[0]}] * {len(values)}'
+                print(f'{item}_{time}\t{value}')
+            else:
+                print(f'{item}_{time}\t{value.strip()}')
+        for time, value in timed_value.items():
+            try:
+                value = eval(value)
+                if len(value) > 1:
+                    value = '{:.3f}'.format(sum(value) / len(value))
+                    print(f'avg_{item}_{time}\t{value}')
+            except:
+                pass
 
 
 class Worker(multiprocessing.Process):
@@ -261,6 +319,7 @@ class Worker(multiprocessing.Process):
     def run(self):
         # set random seed to a random number
         np.random.seed()
+
         while True:
             id = self.task_queue.get()
             if id is None:
@@ -270,7 +329,16 @@ class Worker(multiprocessing.Process):
                 logger.id = id
                 simu = Simulator(
                     params=self.params, logger=logger, simu_args=self.simu_args)
-                simu.simulate(id)
+                try:
+                    simu.simulate(id)
+                except (SystemExit, Exception) as e:
+                    msg = repr(e).replace('\n',
+                                          ' ').replace('\t',
+                                                       ' ').replace(',', ' ')
+                    logger.write(f'{id}\t0.00\tERROR\t.\texception={msg}\n')
+                    self.task_queue.task_done()
+                    self.result_queue.put(logger.getvalue())
+                    raise e
                 self.task_queue.task_done()
                 self.result_queue.put(logger.getvalue())
 
@@ -323,24 +391,21 @@ def parse_args(args=None):
               1 will be assinged to each replicate and as the first columns
               in the log file.''')
     parser.add_argument(
-        '--keep-symptomatic',
-        action='store_true',
-        help='Keep affected individuals in the population')
-    parser.add_argument(
-        '--pre-quarantine',
+        '--handle-symptomatic',
         nargs='*',
-        help='''Days of self-quarantine before introducing infector to the group.
-            The simulation will be aborted if the infector shows symptom before
-            introduction. If you quarantine multiple people or specified named
-            groups, you will need to append the IDs to the parameter (e.g.
-            --pre-quarantine day nurse1 nurse2''')
+        default=['remove'],
+        help='''How to handle individuals who show symptom, which should be "keep" (stay in
+            population), "remove" (remove from population), and "quarantine" (put aside until
+            it recovers). all options can be followed by a "proportion", and quarantine can
+            be specified as "quarantine_7" etc to specify duration of quarantine.'''
+    )
     parser.add_argument(
         '--infectors',
         nargs='*',
-        help='''Infectees to introduce to the population, default to '0'. If you
+        help='''Infectees to introduce to the population. If you
             would like to introduce multiple infectees to the population, or if
             you have named groups, you will have to specify the IDs of carrier
-            such as --infectors nurse1 nurse2''')
+            such as --infectors nurse1 nurse2.''')
     parser.add_argument(
         '--interval',
         default=1 / 24,
@@ -354,6 +419,13 @@ def parse_args(args=None):
         numbers as the lower and higher CI (95%%) of the proportion. Default to 0.10 to 0.40.
         Multipliers can be specified to set proportion of asymptomatic carriers
         for particular groups.''')
+    parser.add_argument(
+        '--stop-if',
+        nargs='*',
+        help='''Condition at which the simulation will end. By default the simulation
+            stops when all individuals are affected or all infected individuals
+            are removed. Current you can specify a time after which the simulation
+            will stop in the format of `--stop-if "t>10"' (for 10 days).''')
     parser.add_argument(
         '--allow-lead-time',
         action='store_true',
@@ -361,10 +433,11 @@ def parse_args(args=None):
             of incurbation time. If allow lead time is set to True, the carrier will
             be anywhere in his or her incubation period.''')
     parser.add_argument(
-        '--analyze-existing-logfile',
-        action='store_true',
-        help='''Analyze an existing logfile, useful for updating the summarization
-            procedure or uncaptured output. ''')
+        '--plugin',
+        nargs=argparse.REMAINDER,
+        help='''One or more of "--plugin MODULE.PLUGIN [args]" to specify one or more
+            plugins. FLUGIN will be assumed to be MODULE name if left unspecified. Each
+            plugin has its own parser and can parse its own args.''')
     parser.add_argument(
         '-j',
         '--jobs',
@@ -374,32 +447,95 @@ def parse_args(args=None):
     return parser.parse_args(args)
 
 
-def main(args=None):
+def main(argv=None):
     """Console script for covid19_outbreak_simulator."""
-    args = parse_args(args)
-    if not args.analyze_existing_logfile:
-        tasks = multiprocessing.JoinableQueue()
-        results = multiprocessing.Queue()
+    args = parse_args(argv)
+    tasks = multiprocessing.JoinableQueue()
+    results = multiprocessing.Queue()
 
-        if not args.jobs:
-            args.jobs = multiprocessing.cpu_count()
+    if args.plugin:
+        if args.plugin[0] == '-h':
+            # list all plugins
+            from covid19_outbreak_simulator.plugin import BasePlugin
+            bp = BasePlugin(simulator=None, population=None)
+            parser = bp.get_parser()
+            parser.parse_args(['-h'])
+        elif args.plugin[0].startswith('-'):
+            # it has to be -h
+            raise ValueError(
+                f'Plugin name should not start with a dash (-): "{args.plugin[0]}" provided.'
+            )
+        else:
+            plugin = args.plugin[0]
+            if '.' not in plugin:
+                module_name, plugin_name = plugin, plugin.replace('-', '_')
+            else:
+                module_name, plugin_name = plugin.rsplit('.', 1)
+            try:
+                mod = import_module(
+                    f'covid19_outbreak_simulator.plugins.{module_name}')
+            except Exception as e:
+                try:
+                    mod = import_module(module_name)
+                except Exception as e:
+                    raise ValueError(
+                        f'Failed to import module {module_name}: {e}')
+            try:
+                obj = getattr(mod, plugin_name)(simulator=None, population=None)
+            except Exception as e:
+                raise ValueError(
+                    f'Failed to retrieve plugin {plugin_name} from module {module_name}: {e}'
+                )
+            # if there is a parser
+            parser = obj.get_parser()
+            if '-h' in args.plugin:
+                parser.parse_args(['-h'])
 
-        workers = [Worker(tasks, results, args) for i in range(args.jobs)]
-        for worker in workers:
-            worker.start()
+    if not args.jobs:
+        args.jobs = multiprocessing.cpu_count()
 
-        with open(args.logfile, 'w') as logger:
-            logger.write('id\ttime\tevent\ttarget\tparams\n')
-            for i in range(args.repeats):
-                tasks.put(i + 1)
-            for i in range(args.jobs):
-                tasks.put(None)
+    if args.stop_if is not None:
+        if args.stop_if[0].startswith('t>'):
             #
-            for i in tqdm(range(args.repeats)):
-                result = results.get()
-                logger.write(result)
+            try:
+                st = float(args.stop_if[0][2:])
+            except Exception:
+                raise ValueError(
+                    f'Invalid value for option --stop-if "{args.stop_if[0]}": {e}'
+                )
+        else:
+            raise ValueError(
+                f'Option --stop-if currently only supports t>TIME to stop after certain time point.'
+            )
 
-    summarize_simulations(args)
+    workers = [
+        Worker(tasks, results, args)
+        for i in range(min(args.jobs, args.repeats))
+    ]
+    for worker in workers:
+        worker.start()
+
+    with open(args.logfile, 'w') as logger:
+        logger.write(
+            f'# CMD: outbreak_simulator {subprocess.list2cmdline(argv if argv else sys.argv[1:])}\n'
+        )
+        logger.write(
+            f'# START: {datetime.now().strftime("%m/%d/%Y, %H:%M:%S")}\n')
+        logger.write('id\ttime\tevent\ttarget\tparams\n')
+        for i in range(args.repeats):
+            tasks.put(i + 1)
+        for i in range(args.jobs):
+            tasks.put(None)
+        #
+        for i in tqdm(range(args.repeats)):
+            result = results.get()
+            logger.write(result)
+            if 'ERROR' in result:
+                break
+        logger.write(
+            f'# START: {datetime.now().strftime("%m/%d/%Y, %H:%M:%S")}\n')
+
+    summarize_simulations(args.logfile)
     return 0
 
 
