@@ -64,6 +64,204 @@ class Params:
             if not found:
                 raise ValueError(f'Invalid ID {ID}')
 
+    def set_popsize(self, val):
+        if not val:
+            return
+        for ps in val:
+            if '=' in ps:
+                name, size = ps.split('=', 1)
+                if name[-1].isnumeric():
+                    raise ValueError(
+                        f'Group names that end with numbers are not allowed. {name}'
+                    )
+            else:
+                name = ''
+                size = int(ps)
+            try:
+                size = int(size)
+            except Exception:
+                raise ValueError(f'Incorrect popsize {ps}')
+            if name in self.groups:
+                raise ValueError(f'Group "{name}" has been specified before')
+            self.groups[name] = size
+
+    def set_infectors(self, val):
+        if not val:
+            return
+        for ID in val:
+            self.check_id(ID)
+
+    def set_symptomatic_r0(self, val):
+        if not val:
+            return
+        # if asymptomatic_r0 is specified, it should a number of a range...
+        pars = [x for x in val if '=' not in x]
+        if len(pars) == 1:
+            try:
+                self.set('symptomatic_r0', 'low', float(pars[0]))
+                self.set('symptomatic_r0', 'high', float(pars[0]))
+            except Exception:
+                raise ValueError(
+                    f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
+                )
+        elif len(pars) == 2:
+            try:
+                self.set('symptomatic_r0', 'low', float(pars[0]))
+            except Exception as e:
+                raise ValueError(
+                    f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
+                )
+            try:
+                self.set('symptomatic_r0', 'high', float(pars[1]))
+            except Exception as e:
+                raise ValueError(
+                    f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[1]} provided'
+                )
+        elif len(pars) > 2:
+            raise ValueError(
+                f'The symptomatic_r0 should be one or two float number.')
+        #
+        for multiplier in [x for x in val if '=' in x]:
+            name, value = multiplier.split('=', 1)
+            try:
+                value = float(value)
+            except Exception:
+                raise ValueError(
+                    f'Multiplier should have format name=float_value: {multiplier} provided'
+                )
+            self.set('symptomatic_r0', f'multiplier_{name}', value)
+
+    def set_asymptomatic_r0(self, val):
+        if not val:
+            return
+        # if asymptomatic_r0 is specified, it should a number of a range...
+        pars = [x for x in val if '=' not in x]
+        if len(pars) == 1:
+            try:
+                self.set('asymptomatic_r0', 'low', float(pars[0]))
+                self.set('asymptomatic_r0', 'high', float(pars[0]))
+            except Exception:
+                raise ValueError(
+                    f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
+                )
+        elif len(pars) == 2:
+            try:
+                self.set('asymptomatic_r0', 'low', float(pars[0]))
+            except Exception as e:
+                raise ValueError(
+                    f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
+                )
+            try:
+                self.set('asymptomatic_r0', 'high', float(pars[1]))
+            except Exception as e:
+                raise ValueError(
+                    f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[1]} provided'
+                )
+        elif len(pars) > 2:
+            raise ValueError(
+                f'The asymptomatic_r0 should be one or two float number.')
+        #
+        for multiplier in [x for x in val if '=' in x]:
+            name, value = multiplier.split('=', 1)
+            try:
+                value = float(value)
+            except Exception:
+                raise ValueError(
+                    f'Multiplier should have format name=float_value: {multiplier} provided'
+                )
+            self.set('asymptomatic_r0', f'multiplier_{name}', value)
+
+    def set_incubation_period(self, val):
+        if not val:
+            return
+
+        pars = [x for x in val if x in ('normal', 'lognormal') or '=' not in x]
+        if pars:
+            if len(pars) < 3:
+                raise ValueError(
+                    f'Parameter incubation period requires aat least three values: {len(val)} provided'
+                )
+            if pars[0] not in ('normal', 'lognormal'):
+                raise ValueError(
+                    f'Only normal or lognormal distribution for incubation period is supported. {pars[0]} provided'
+                )
+            try:
+                self.set('incubation_period',
+                         'mean' if pars[0] == 'lognormal' else 'loc',
+                         float(pars[1]))
+            except Exception:
+                raise ValueError(
+                    f'Second parameter of incubation_period should be a float number: {pars[1]} provided'
+                )
+            try:
+                self.set('incubation_period',
+                         'sigma' if pars[0] == 'lognormal' else 'scale',
+                         float(pars[2]))
+            except Exception:
+                raise ValueError(
+                    f'Third parameter of lognormal incubation_period should be a float number: {pars[2]} provided'
+                )
+        # multipliers
+        for multiplier in [x for x in val if '=' in x]:
+            name, value = multiplier.split('=', 1)
+            try:
+                value = float(value)
+            except Exception:
+                raise ValueError(
+                    f'Multiplier should have format name=float_value: {multiplier} provided'
+                )
+            self.set('incubation_period', f'multiplier_{name}', value)
+
+    def set_susceptibility(self, val):
+        if not val:
+            return
+        for multiplier in val:
+            if '=' not in multiplier:
+                raise ValueError(
+                    f'Susceptibility has to be specified as name=weight: {multiplier} specified.'
+                )
+            name, value = multiplier.split('=', 1)
+            try:
+                value = float(value)
+            except Exception:
+                raise ValueError(
+                    f'Multiplier should have format name=float_value: {multiplier} provided'
+                )
+            self.set('susceptibility', f'multiplier_{name}', value)
+
+    def set_prop_asym_carriers(self, val):
+        if not val:
+            return
+        try:
+            pars = [float(x) for x in val if '=' not in x]
+        except Exception:
+            raise ValueError(
+                f'Paramter prop-asym-carriers expect one or two float value or multipliers: {" ".join(val)} provided'
+            )
+        if len(pars) == 1:
+            self.set('prop_asym_carriers', 'loc', float(pars[0]))
+            self.set('prop_asym_carriers', 'scale', 0)
+        elif len(pars) == 2:
+            if pars[0] > pars[1]:
+                raise ValueError(
+                    f'Proportions for parameter prop-asym-carriers should be incremental.'
+                )
+            self.set('prop_asym_carriers', 'loc', (pars[0] + pars[1]) / 2)
+            self.set('prop_asym_carriers', 'quantile_2.5', pars[0])
+        elif len(pars) > 2:
+            raise ValueError(
+                f'Parameter prop-asym-carriers accepts one or two numbers.')
+        #
+        for multiplier in [x for x in val if '=' in x]:
+            name, value = multiplier.split('=', 1)
+            try:
+                value = float(value)
+            except Exception:
+                raise ValueError(
+                    f'Multiplier should have format name=float_value: {multiplier} provided'
+                )
+            self.set('prop_asym_carriers', f'multiplier_{name}', value)
+
     def set_params(self, args):
         # set some default values first
         self.set('simulation_interval', 'self',
@@ -79,202 +277,15 @@ class Params:
 
         if not args:
             return
-        if args.popsize:
-            for ps in args.popsize:
-                if '=' in ps:
-                    name, size = ps.split('=', 1)
-                    if name[-1].isnumeric():
-                        raise ValueError(
-                            f'Group names that end with numbers are not allowed. {name}'
-                        )
-                else:
-                    name = ''
-                    size = int(ps)
-                try:
-                    size = int(size)
-                except Exception:
-                    raise ValueError(f'Incorrect popsize {ps}')
-                if name in self.groups:
-                    raise ValueError(
-                        f'Group "{name}" has been specified before')
-                self.groups[name] = size
-
-        # do a favor to check parameters, but does not really save parameters here
-        if args.pre_quarantine:
-            try:
-                float(args.pre_quarantine[0])
-            except Exception:
-                raise ValueError(
-                    'The first parameter of pre_quarantine should be a number')
-            for ID in args.pre_quarantine[1:]:
-                self.check_id(ID)
-
-        # do a favor to check parameters, but does not really save parameters here
-        if args.infectors:
-            for ID in args.infectors:
-                self.check_id(ID)
 
+        self.set_popsize(args.popsize)
+        self.set_infectors(args.infectors)
         # modify from command line args
-        if args.symptomatic_r0:
-            # if asymptomatic_r0 is specified, it should a number of a range...
-            pars = [x for x in args.symptomatic_r0 if '=' not in x]
-            if len(pars) == 1:
-                try:
-                    self.set('symptomatic_r0', 'low', float(pars[0]))
-                    self.set('symptomatic_r0', 'high', float(pars[0]))
-                except Exception:
-                    raise ValueError(
-                        f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                    )
-            elif len(pars) == 2:
-                try:
-                    self.set('symptomatic_r0', 'low', float(pars[0]))
-                except Exception as e:
-                    raise ValueError(
-                        f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                    )
-                try:
-                    self.set('symptomatic_r0', 'high', float(pars[1]))
-                except Exception as e:
-                    raise ValueError(
-                        f'The symptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[1]} provided'
-                    )
-            elif len(pars) > 2:
-                raise ValueError(
-                    f'The symptomatic_r0 should be one or two float number.')
-            #
-            for multiplier in [x for x in args.symptomatic_r0 if '=' in x]:
-                name, value = multiplier.split('=', 1)
-                try:
-                    value = float(value)
-                except Exception:
-                    raise ValueError(
-                        f'Multiplier should have format name=float_value: {multiplier} provided'
-                    )
-                self.set('symptomatic_r0', f'multiplier_{name}', value)
-        if args.asymptomatic_r0:
-            # if asymptomatic_r0 is specified, it should a number of a range...
-            pars = [x for x in args.asymptomatic_r0 if '=' not in x]
-            if len(pars) == 1:
-                try:
-                    self.set('asymptomatic_r0', 'low', float(pars[0]))
-                    self.set('asymptomatic_r0', 'high', float(pars[0]))
-                except Exception:
-                    raise ValueError(
-                        f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                    )
-            elif len(pars) == 2:
-                try:
-                    self.set('asymptomatic_r0', 'low', float(pars[0]))
-                except Exception as e:
-                    raise ValueError(
-                        f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[0]} provided'
-                    )
-                try:
-                    self.set('asymptomatic_r0', 'high', float(pars[1]))
-                except Exception as e:
-                    raise ValueError(
-                        f'The asymptomatic_r0 should be a float number, if it is not a multiplier for groups: {pars[1]} provided'
-                    )
-            elif len(pars) > 2:
-                raise ValueError(
-                    f'The asymptomatic_r0 should be one or two float number.')
-            #
-            for multiplier in [x for x in args.asymptomatic_r0 if '=' in x]:
-                name, value = multiplier.split('=', 1)
-                try:
-                    value = float(value)
-                except Exception:
-                    raise ValueError(
-                        f'Multiplier should have format name=float_value: {multiplier} provided'
-                    )
-                self.set('asymptomatic_r0', f'multiplier_{name}', value)
-        if args.incubation_period:
-            pars = [
-                x for x in args.incubation_period
-                if x in ('normal', 'lognormal') or '=' not in x
-            ]
-            if pars:
-                if len(pars) < 3:
-                    raise ValueError(
-                        f'Parameter incubation period requires aat least three values: {len(args.incubation_period)} provided'
-                    )
-                if pars[0] not in ('normal', 'lognormal'):
-                    raise ValueError(
-                        f'Only normal or lognormal distribution for incubation period is supported. {pars[0]} provided'
-                    )
-                try:
-                    self.set('incubation_period',
-                             'mean' if pars[0] == 'lognormal' else 'loc',
-                             float(pars[1]))
-                except Exception:
-                    raise ValueError(
-                        f'Second parameter of incubation_period should be a float number: {pars[1]} provided'
-                    )
-                try:
-                    self.set('incubation_period',
-                             'sigma' if pars[0] == 'lognormal' else 'scale',
-                             float(pars[2]))
-                except Exception:
-                    raise ValueError(
-                        f'Third parameter of lognormal incubation_period should be a float number: {pars[2]} provided'
-                    )
-            # multipliers
-            for multiplier in [x for x in args.incubation_period if '=' in x]:
-                name, value = multiplier.split('=', 1)
-                try:
-                    value = float(value)
-                except Exception:
-                    raise ValueError(
-                        f'Multiplier should have format name=float_value: {multiplier} provided'
-                    )
-                self.set('incubation_period', f'multiplier_{name}', value)
-        if args.susceptibility:
-            for multiplier in args.susceptibility:
-                if '=' not in multiplier:
-                    raise ValueError(
-                        f'Susceptibility has to be specified as name=weight: {multiplier} specified.'
-                    )
-                name, value = multiplier.split('=', 1)
-                try:
-                    value = float(value)
-                except Exception:
-                    raise ValueError(
-                        f'Multiplier should have format name=float_value: {multiplier} provided'
-                    )
-                self.set('susceptibility', f'multiplier_{name}', value)
-        if args.prop_asym_carriers:
-            try:
-                pars = [
-                    float(x) for x in args.prop_asym_carriers if '=' not in x
-                ]
-            except Exception:
-                raise ValueError(
-                    f'Paramter prop-asym-carriers expect one or two float value or multipliers: {" ".join(args.prop_asym_carriers)} provided'
-                )
-            if len(pars) == 1:
-                self.set('prop_asym_carriers', 'loc', float(pars[0]))
-                self.set('prop_asym_carriers', 'scale', 0)
-            elif len(pars) == 2:
-                if pars[0] > pars[1]:
-                    raise ValueError(
-                        f'Proportions for parameter prop-asym-carriers should be incremental.'
-                    )
-                self.set('prop_asym_carriers', 'loc', (pars[0] + pars[1]) / 2)
-                self.set('prop_asym_carriers', 'quantile_2.5', pars[0])
-            elif len(pars) > 2:
-                raise ValueError(
-                    f'Parameter prop-asym-carriers accepts one or two numbers.')
-            #
-            for multiplier in [x for x in args.prop_asym_carriers if '=' in x]:
-                name, value = multiplier.split('=', 1)
-                try:
-                    value = float(value)
-                except Exception:
-                    raise ValueError(
-                        f'Multiplier should have format name=float_value: {multiplier} provided'
-                    )
-                self.set('prop_asym_carriers', f'multiplier_{name}', value)
+        self.set_symptomatic_r0(args.symptomatic_r0)
+        self.set_asymptomatic_r0(args.asymptomatic_r0)
+        self.set_incubation_period(args.incubation_period)
+        self.set_susceptibility(args.susceptibility)
+        self.set_prop_asym_carriers(args.prop_asym_carriers)
 
 
 class Model(object):
