@@ -1,9 +1,9 @@
-
 import os
 import pytest
 from covid19_outbreak_simulator.cli import parse_args, main
 from covid19_outbreak_simulator.model import Params
 
+
 def test_option_popsize():
     args = parse_args(['--popsize', '500'])
     params = Params(args)
@@ -33,8 +33,10 @@ def test_option_popsize():
         args = parse_args(['--popsize', '500', 'A6=300'])
         params = Params(args)
 
+
 def test_option_susceptibility():
-    args = parse_args(['--popsize', 'A=500', 'B=300', '--susceptibility', 'A=0.8'])
+    args = parse_args(
+        ['--popsize', 'A=500', 'B=300', '--susceptibility', 'A=0.8'])
     params = Params(args)
     assert sorted(list(params.groups.keys())) == ['A', 'B']
 
@@ -44,14 +46,17 @@ def test_option_susceptibility():
 
     # group does not exist
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=500', 'B=300', '--susceptibility', 'C=0.8'])
+        args = parse_args(
+            ['--popsize', 'A=500', 'B=300', '--susceptibility', 'C=0.8'])
         params = Params(args)
 
     # not as multiplier
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=500', 'B=300', '--susceptibility', '0.8'])
+        args = parse_args(
+            ['--popsize', 'A=500', 'B=300', '--susceptibility', '0.8'])
         params = Params(args)
 
+
 def test_option_symptomatic_r0():
     args = parse_args(['--symptomatic-r0', '1.0'])
     params = Params(args)
@@ -63,8 +68,8 @@ def test_option_symptomatic_r0():
     assert params.symptomatic_r0_low == 1.2
     assert params.symptomatic_r0_high == 2.0
 
-    args = parse_args(['--popsize', 'A=500', 'B=300',
-         '--symptomatic-r0', '1.5', 'A=0.8'])
+    args = parse_args(
+        ['--popsize', 'A=500', 'B=300', '--symptomatic-r0', '1.5', 'A=0.8'])
     params = Params(args)
     assert sorted(list(params.groups.keys())) == ['A', 'B']
     assert params.symptomatic_r0_low == 1.5
@@ -89,6 +94,7 @@ def test_option_symptomatic_r0():
         args = parse_args(['--symptomatic-r0', '0.8', '1.0', '2.0'])
         params = Params(args)
 
+
 def test_option_asymptomatic_r0():
     args = parse_args(['--asymptomatic-r0', '1.0'])
     params = Params(args)
@@ -100,8 +106,8 @@ def test_option_asymptomatic_r0():
     assert params.asymptomatic_r0_low == 1.2
     assert params.asymptomatic_r0_high == 2.0
 
-    args = parse_args(['--popsize', 'A=500', 'B=300',
-         '--asymptomatic-r0', '1.5', 'A=0.8'])
+    args = parse_args(
+        ['--popsize', 'A=500', 'B=300', '--asymptomatic-r0', '1.5', 'A=0.8'])
     params = Params(args)
     assert sorted(list(params.groups.keys())) == ['A', 'B']
     assert params.asymptomatic_r0_low == 1.5
@@ -126,6 +132,7 @@ def test_option_asymptomatic_r0():
         args = parse_args(['--asymptomatic-r0', '0.8', '1.0', '2.0'])
         params = Params(args)
 
+
 def test_option_inclubation_period():
     with pytest.raises(ValueError):
         args = parse_args(['--incubation-period', '1.0'])
@@ -140,8 +147,8 @@ def test_option_inclubation_period():
         params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=100', 'B=200',
-            '--incubation-period', 'C=0.8'])
+        args = parse_args(
+            ['--popsize', 'A=100', 'B=200', '--incubation-period', 'C=0.8'])
         params = Params(args)
 
     args = parse_args(['--incubation-period', 'normal', '1.2', '2.0'])
@@ -154,14 +161,16 @@ def test_option_inclubation_period():
     assert params.incubation_period_mean == 1.
     assert params.incubation_period_sigma == 2.5
 
-    args = parse_args(['--popsize', 'A=500', 'B=300',
-         '--incubation-period', 'A=0.8'])
+    args = parse_args(
+        ['--popsize', 'A=500', 'B=300', '--incubation-period', 'A=0.8'])
     params = Params(args)
     assert sorted(list(params.groups.keys())) == ['A', 'B']
     assert params.incubation_period_multiplier_A == 0.8
 
-    args = parse_args(['--popsize', 'A=500', 'B=300',
-        '--incubation-period', 'normal', '5', '2', 'B=0.95', 'A=1.13'])
+    args = parse_args([
+        '--popsize', 'A=500', 'B=300', '--incubation-period', 'normal', '5',
+        '2', 'B=0.95', 'A=1.13'
+    ])
     params = Params(args)
     assert sorted(list(params.groups.keys())) == ['A', 'B']
     assert params.incubation_period_loc == 5.0
@@ -170,79 +179,47 @@ def test_option_inclubation_period():
     assert params.incubation_period_multiplier_B == 0.95
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=500', 'B=300',
-            '--incubation-period', 'normal', '5', '2.3p'])
+        args = parse_args([
+            '--popsize', 'A=500', 'B=300', '--incubation-period', 'normal', '5',
+            '2.3p'
+        ])
         params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=500', 'B=300',
-            '--incubation-period', 'normal', '5', '2.3p', 'A=1.4p'])
+        args = parse_args([
+            '--popsize', 'A=500', 'B=300', '--incubation-period', 'normal', '5',
+            '2.3p', 'A=1.4p'
+        ])
         params = Params(args)
 
     # negative multiplier
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=500', 'B=300',
-            '--incubation-period', 'A=-1'])
+        args = parse_args(
+            ['--popsize', 'A=500', 'B=300', '--incubation-period', 'A=-1'])
         params = Params(args)
 
-def test_option_pre_quarantine():
-    args = parse_args(['--pre-quarantine', '10'])
-    params = Params(args)
-
-    args = parse_args(['--popsize', 'A=10', 'B=10',
-         '--pre-quarantine', '10', 'A1', 'B2'])
-    params = Params(args)
-
-    with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--pre-quarantine', '10', '1'])
-        params = Params(args)
-
-    # index out of range
-    with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--pre-quarantine', '10', 'C1', 'C2'])
-        params = Params(args)
-
-    # missing quarantine time
-    with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--pre-quarantine', 'A1', 'A2'])
-        params = Params(args)
-
-    with pytest.raises(ValueError):
-        args = parse_args(['--popsize', '100',
-             '--pre-quarantine', '10', '101'])
-        params = Params(args)
-
-    with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--pre-quarantine', '10', 'A10'])
-        params = Params(args)
 
 def test_option_infectors():
     args = parse_args(['--infectors', '10'])
     params = Params(args)
 
-    args = parse_args(['--popsize', 'A=10', 'B=10',
-         '--infectors', 'A1', 'B2'])
+    args = parse_args(['--popsize', 'A=10', 'B=10', '--infectors', 'A1', 'B2'])
     params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--infectors', '1'])
+        args = parse_args(['--popsize', 'A=10', 'B=10', '--infectors', '1'])
         params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--infectors', 'C1', 'C2'])
+        args = parse_args(
+            ['--popsize', 'A=10', 'B=10', '--infectors', 'C1', 'C2'])
         params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--infectors', 'A10'])
+        args = parse_args(['--popsize', 'A=10', 'B=10', '--infectors', 'A10'])
         params = Params(args)
 
+
 def test_option_prop_asym_carriers():
     args = parse_args(['--prop-asym-carriers', '0.1'])
     params = Params(args)
@@ -254,48 +231,62 @@ def test_option_prop_asym_carriers():
     assert params.prop_asym_carriers_loc == 0.2
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--prop-asym-carriers', 'C=1.2'])
+        args = parse_args(
+            ['--popsize', 'A=10', 'B=10', '--prop-asym-carriers', 'C=1.2'])
         params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--prop-asym-carriers', 'A=-1.2'])
+        args = parse_args(
+            ['--popsize', 'A=10', 'B=10', '--prop-asym-carriers', 'A=-1.2'])
         params = Params(args)
 
     with pytest.raises(ValueError):
-        args = parse_args(['--popsize', 'A=10', 'B=10',
-             '--prop-asym-carriers', 'A=1.2f'])
+        args = parse_args(
+            ['--popsize', 'A=10', 'B=10', '--prop-asym-carriers', 'A=1.2f'])
         params = Params(args)
 
-    args = parse_args(['--popsize', 'A=10', 'B=10',
-            '--prop-asym-carriers', 'A=1.2'])
+    args = parse_args(
+        ['--popsize', 'A=10', 'B=10', '--prop-asym-carriers', 'A=1.2'])
     params = Params(args)
     assert params.prop_asym_carriers_multiplier_A == 1.2
 
+
 def test_main_default():
     main(['--jobs', '1', '--repeats', '100'])
     main(['--jobs', '1', '--repeats', '100', '--logfile', 'test.out'])
     assert os.path.isfile('test.out')
-    main(['--analyze-existing-logfile', '--logfile', 'test.out'])
+    main(['--logfile', 'test.out'])
+
 
 def test_main_symptomatic():
     main(['--jobs', '1', '--repeats', '100', '--symptomatic-r0', '1.0'])
     main(['--jobs', '1', '--repeats', '100', '--symptomatic-r0', '1.0', '2.0'])
 
+
 def test_main_asymptomatic():
     main(['--jobs', '1', '--repeats', '100', '--asymptomatic-r0', '0.5'])
     main(['--jobs', '1', '--repeats', '100', '--asymptomatic-r0', '0.5', '1.5'])
-    main(['--jobs', '1', '--repeats', '100', '--popsize', 'A=100', 'B=50',
-        '--asymptomatic-r0', '0.5', '1.5', 'A=1.2', '--infector', 'A0'])
+    main([
+        '--jobs', '1', '--repeats', '100', '--popsize', 'A=100', 'B=50',
+        '--asymptomatic-r0', '0.5', '1.5', 'A=1.2', '--infector', 'A0'
+    ])
 
-def test_main_pre_quarantine():
-    main(['--jobs', '1', '--repeats', '100', '--pre-quarantine', '7'])
-    main(['--jobs', '1', '--repeats', '100', '--pre-quarantine', '7', '1', '2'])
-    main(['--jobs', '1', '--repeats', '100', '--popsize', 'A=100', 'B=200',
-        '--pre-quarantine', '7', 'A2', 'A7',  '--infector', 'A2', 'A7'])
 
 def test_main_incubation_period():
-    main(['--jobs', '1', '--repeats', '100', '--incubation-period', 'lognormal', '1', '0.2'])
-    main(['--jobs', '1', '--repeats', '100', '--incubation-period', 'normal', '1', '2'])
+    main([
+        '--jobs', '1', '--repeats', '100', '--incubation-period', 'lognormal',
+        '1', '0.2'
+    ])
+    main([
+        '--jobs', '1', '--repeats', '100', '--incubation-period', 'normal', '1',
+        '2'
+    ])
 
+
+def test_main_stop_if():
+    main(['--jobs', '1', '--repeats', '100', '--stop-if', 't>1'])
+
+
+def test_main_stop_if_error():
+    with pytest.raises(Exception):
+        main(['--jobs', '1', '--repeats', '100', '--stop-if', 'st>1'])
